#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Inspire by LittleWebWut tutorial found here: https://www.littlewebhut.com/gimp/

# author: Pocholo
# date: 10/22/22

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugin folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

# Change log:
#   Version 2
#   10-10-2022
#   Use GIMP only resources 
#   1. Now the branches are created by the Mosaic pligin
#   2. The Mossy bark is created using GIMP's own pattern Tree Bark


from gimpfu import *
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def wired_text_effect(customtext, fontname, shape, colorize, color):
		
	#Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
	
    size = 175
    img = pdb.gimp_image_new(1, 1, RGB)
    pdb.gimp_context_set_foreground("#000000")
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 100, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_line_spacing(textLayer, 0)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 10)
    #Resize text
    img.resize(textLayer.width, textLayer.height, 0, 0)
    #Rename the text layer and move up
    pdb.gimp_item_set_name(textLayer, "Text")
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    x = textLayer.offsets[0]
    y = textLayer.offsets[1]
    x = (img.width - textLayer.width) / 2
    y = (img.height - textLayer.height) / 2
    textLayer.set_offsets(x, y)
    pdb.gimp_layer_resize_to_image_size(textLayer)
	
    #Image process
    gimp.Display(img)  
    
    #Otions for round or square shape
    wireLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Wire", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, wireLayer, None, 1)
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_drawable_fill(wireLayer, FILL_TRANSPARENT)
    
    if shape==0:
        pdb.gimp_image_select_ellipse(img, CHANNEL_OP_REPLACE, 10, 10, img.width-20, img.height-22)
        pdb.gimp_selection_feather(img, 5)
        pdb.plug_in_sel2path(img, wireLayer)
        pdb.gimp_context_set_line_width(10)
        pdb.gimp_context_set_line_width_unit(PIXELS)
        pdb.gimp_context_set_stroke_method (STROKE_LINE)
        vectors = pdb.gimp_image_get_vectors_by_name(img, _("Selection"))
        pdb.gimp_drawable_edit_stroke_item(wireLayer, vectors)
        pdb.plug_in_grid(img, wireLayer, 
                                        5,    #hwidth
                                       23,    #hspace
                                        9,    #hoffset
                                "#000000",    #hcolor
                                      255,    #hopacity
                                        5,    #vwidth
                                       23,    #vspace
                                        9,    #voffset
                                "#000000",    #vcolor
                                      255,    #vopacity
                                        0,    #iwidth
                                        2,    #ispac
                                        6,    #ioffset
                                "#000000",    #icolor
                                      255)    #iopacity
    
    if shape==1:
        pdb.gimp_image_select_rectangle(img, CHANNEL_OP_REPLACE, 0, 0, img.width, img.height)
        pdb.gimp_selection_feather(img, 5)
        pdb.plug_in_sel2path(img, wireLayer)
        pdb.gimp_context_set_line_width(10)
        pdb.gimp_context_set_line_width_unit(PIXELS)
        pdb.gimp_context_set_stroke_method (STROKE_LINE)
        vectors = pdb.gimp_image_get_vectors_by_name(img, _("Selection"))
        pdb.gimp_drawable_edit_stroke_item(wireLayer, vectors)
        pdb.plug_in_grid(img, wireLayer, 
                                        5,    #hwidth
                                       23,    #hspace
                                        9,    #hoffset
                                "#000000",    #hcolor
                                      255,    #hopacity
                                        5,    #vwidth
                                       23,    #vspace
                                        9,    #voffset
                                "#000000",    #vcolor
                                      255,    #vopacity
                                        0,    #iwidth
                                        2,    #ispac
                                        6,    #ioffset
                                "#000000",    #icolor
                                      255)    #iopacity
    
    
    pdb.gimp_selection_none(img)
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_selection_feather(img, 8)
    wireLayer = pdb.gimp_image_merge_down(img, textLayer, CLIP_TO_IMAGE)  
    pdb.gimp_selection_shrink(img, 4)
    pdb.gimp_drawable_edit_clear(wireLayer)
    pdb.gimp_selection_none(img)
    
    #Add three white layers and a shadow layer
    pdb.gimp_item_set_visible(wireLayer, FALSE)
    layer_1 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Layer #1", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, layer_1, None, 1)
    pdb.gimp_drawable_fill(layer_1, FILL_WHITE)
    layer_2 = pdb.gimp_layer_new_from_drawable(layer_1, img)
    pdb.gimp_image_insert_layer(img, layer_2, None, 1)
    pdb.gimp_layer_set_mode(layer_2, LAYER_MODE_DIFFERENCE_LEGACY)
    pdb.gimp_layer_set_opacity(layer_2, 100)
    layer_3 = pdb.gimp_layer_new_from_drawable(layer_2, img)
    pdb.gimp_image_insert_layer(img, layer_3, None, 1)
    pdb.gimp_layer_set_mode(layer_3, LAYER_MODE_DIFFERENCE_LEGACY)
    pdb.gimp_layer_set_opacity(layer_3, 100)
    shadLayer = pdb.gimp_layer_new_from_drawable(layer_3, img)
    pdb.gimp_image_insert_layer(img, shadLayer, None, 4)
    pdb.gimp_layer_set_mode(shadLayer, LAYER_MODE_NORMAL)
    pdb.gimp_layer_set_opacity(shadLayer, 100)
    pdb.gimp_item_set_name(shadLayer, "Drop Shadow")
    pdb.gimp_drawable_edit_clear(shadLayer)
    pdb.gimp_item_set_visible(shadLayer, FALSE)
    
    #Chrome the image
    offx1 = 2
    offy1 = 2
    offx2 = -4
    offy2 = -4
    feather =4
    shadfeather = 6
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_image_select_item(img, 2, wireLayer)
    pdb.gimp_edit_bucket_fill(layer_3, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_translate(img, offx1, offy1)
    pdb.gimp_selection_feather(img, feather)
    pdb.gimp_edit_bucket_fill(layer_2, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_translate(img, offx2, offy2)
    pdb.gimp_edit_bucket_fill(layer_1, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_none(img)
    
    #Add a mask to make it metalic
    chroLayer = pdb.gimp_image_merge_visible_layers(img, EXPAND_AS_NECESSARY)
    pdb.gimp_drawable_invert(chroLayer, FALSE)
    mask = pdb.gimp_layer_create_mask(chroLayer, ADD_MASK_BLACK)
    pdb.gimp_layer_add_mask(chroLayer, mask)
    pdb.gimp_image_select_item(img, 2, wireLayer)
    pdb.gimp_context_set_foreground("#ffffff")
    pdb.gimp_edit_bucket_fill(mask, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_layer_set_apply_mask(chroLayer, TRUE)
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_item_set_visible(shadLayer, TRUE)
    pdb.gimp_selection_translate(img, offx1, offy1)
    pdb.gimp_selection_feather(img, shadfeather)
    pdb.gimp_edit_bucket_fill(shadLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_none(img)
    pdb.gimp_levels(chroLayer, 
                            0,   #channel - histogram-alpha
                            0,   #low_input 
                          255,   #high_input 
                         0.30,   #gamma 
                            0,   #low_output
                          255)   #high_output 
    
    #Color choice
    if colorize:
        colorLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Color", 100, LAYER_MODE_HSL_COLOR)
        pdb.gimp_image_insert_layer(img, colorLayer, None, 0)
        pdb.gimp_context_set_foreground(color)
        pdb.gimp_drawable_fill(colorLayer, FILL_FOREGROUND)
        
    #pdb.gimp_item_set_visible(backLayer, TRUE)
    pdb.gimp_image_remove_layer(img, wireLayer)
    
    #Resets the previous user defaults
    pdb.gimp_context_pop()	
	
register(
    "wired_text_effect",
    "Creates a Tree bark text",
    "Creates a Tree bark text text in a frame",
    "Pocholo",
    "Pocholo",
    "2020",
    "Create a Wired Text",
    "",
    [
    (PF_TEXT, "customtext", "Text string", "WIRED\nTEXT"),
    (PF_FONT, "fontname", "Font name", "Arial Bold"),
    (PF_OPTION, "shape", "Wire Shapes:", 0, ("Round", "Square")),
    (PF_TOGGLE, "colorize", "Colorize?:", 0),
    (PF_COLOR, "color", "Wire Color", (155, 250, 250)),
    ],
    [], 
    wired_text_effect, menu="<Image>/Python-Fu/Text Effects",
    domain=("gimp20", gimp.locale_directory))
main()			    